if not LegendaryCursorSkins then
	_G.LegendaryCursorSkins = {}
	LegendaryCursorSkins.mod_path = ModPath
	LegendaryCursorSkins.save_path = SavePath
	LegendaryCursorSkins.settings = {
		id = "default",
		red = 0.7,
		green = 0.7,
		blue = 0.7,
		scale = 1
	}

	function LegendaryCursorSkins:save()
		local file = io.open(self.save_path .. "legendary_cursor_skins.json", "w+")
		if file then
			file:write(json.encode(self.settings))
			file:close()
		end
	end

	function LegendaryCursorSkins:load()
		local file = io.open(self.save_path .. "legendary_cursor_skins.json", "r")
		if file then
			local data = json.decode(file:read("*all")) or {}
			file:close()
			for k, v in pairs(data) do
				self.settings[k] = v
			end
		end
	end

	LegendaryCursorSkins:load()
end

local menu_id_main = "LegendaryCursorSkinsMenu"
Hooks:Add("MenuManagerSetupCustomMenus", "LegendaryCursorSkins_MenuManagerSetupCustomMenus", function(menu_manager, nodes)
  MenuHelper:NewMenu(menu_id_main)
end)

Hooks:Add("MenuManagerPopulateCustomMenus", "LegendaryCursorSkins_MenuManagerPopulateCustomMenus", function(menu_manager, nodes)
	if not tweak_data.cursors[LegendaryCursorSkins.settings.id] then
		LegendaryCursorSkins.settings.id = "default"
	end

	local cursors = {"default"}
	local cursor_names = {"menu_default"}
	local current_index = 1
	local initial_save_value = 1
	for index, cursor in pairs(tweak_data.cursors) do
		if index ~= "default" then
			current_index = current_index + 1

			if index == LegendaryCursorSkins.settings.id then
				initial_save_value = current_index
			end

			table.insert(cursors, index)
			table.insert(cursor_names, cursor.name_id or ("menu_cursor_" .. index))
		end
	end

	MenuCallbackHandler.menu_cursor_selection_callback = function(self, item)
		local cursor_id = cursors[item:value()]
		LegendaryCursorSkins.settings.id = cursor_id
		LegendaryCursorSkins:save()
		managers.mouse_pointer:set_cursor(cursor_id)
	end

	MenuHelper:AddMultipleChoice({
	    id = "cursor_selection",
	    title = "menu_cursor_selection",
	    desc = "menu_cursor_selection_desc",
	    callback = "menu_cursor_selection_callback",
	    items = cursor_names,
	    value = initial_save_value,
	    menu_id = menu_id_main,
	    priority = 9,
	})

	MenuHelper:AddDivider({
		id = "divider1",
		size = 24,
		menu_id = menu_id_main,
		priority = 8
	})

	MenuCallbackHandler.menu_cursor_red_callback = function(self, item)
		local new_red = item:value()
		LegendaryCursorSkins.settings.red = new_red
		LegendaryCursorSkins:save()

		managers.mouse_pointer:set_user_properties(Color(1, LegendaryCursorSkins.settings.red, LegendaryCursorSkins.settings.green, LegendaryCursorSkins.settings.blue), LegendaryCursorSkins.settings.scale)
	end

	MenuCallbackHandler.menu_cursor_green_callback = function(self, item)
		local new_green = item:value()
		LegendaryCursorSkins.settings.green = new_green
		LegendaryCursorSkins:save()

		managers.mouse_pointer:set_user_properties(Color(1, LegendaryCursorSkins.settings.red, LegendaryCursorSkins.settings.green, LegendaryCursorSkins.settings.blue), LegendaryCursorSkins.settings.scale)
	end

	MenuCallbackHandler.menu_cursor_blue_callback = function(self, item)
		local new_blue = item:value()
		LegendaryCursorSkins.settings.blue = new_blue
		LegendaryCursorSkins:save()

		managers.mouse_pointer:set_user_properties(Color(1, LegendaryCursorSkins.settings.red, LegendaryCursorSkins.settings.green, LegendaryCursorSkins.settings.blue), LegendaryCursorSkins.settings.scale)
	end

	MenuHelper:AddSlider({
		id = "red",
		title = "menu_cursor_red",
		desc = "menu_cursor_red_desc",
		callback = "menu_cursor_red_callback",
		value = LegendaryCursorSkins.settings.red,
		min = 0,
		max = 1,
		step = 0.01,
		show_value = true,
		menu_id = menu_id_main,
		priority = 7
	})

	MenuHelper:AddSlider({
		id = "green",
		title = "menu_cursor_green",
		desc = "menu_cursor_green_desc",
		callback = "menu_cursor_green_callback",
		value = LegendaryCursorSkins.settings.green,
		min = 0,
		max = 1,
		step = 0.01,
		show_value = true,
		menu_id = menu_id_main,
		priority = 6
	})

	MenuHelper:AddSlider({
		id = "blue",
		title = "menu_cursor_blue",
		desc = "menu_cursor_blue_desc",
		callback = "menu_cursor_blue_callback",
		value = LegendaryCursorSkins.settings.blue,
		min = 0,
		max = 1,
		step = 0.01,
		show_value = true,
		menu_id = menu_id_main,
		priority = 5
	})

	MenuHelper:AddDivider({
		id = "divider2",
		size = 24,
		menu_id = menu_id_main,
		priority = 4
	})

	MenuCallbackHandler.menu_cursor_scale_callback = function(self, item)
		local new_scale = item:value()
		LegendaryCursorSkins.settings.scale = new_scale
		LegendaryCursorSkins:save()

		managers.mouse_pointer:set_user_properties(Color(1, LegendaryCursorSkins.settings.red, LegendaryCursorSkins.settings.green, LegendaryCursorSkins.settings.blue), LegendaryCursorSkins.settings.scale)
	end

	MenuHelper:AddSlider({
		id = "scale",
		title = "menu_cursor_scale",
		desc = "menu_cursor_scale_desc",
		callback = "menu_cursor_scale_callback",
		value = LegendaryCursorSkins.settings.scale,
		min = 1,
		max = 10,
		step = 0.5,
		show_value = true,
		menu_id = menu_id_main,
		priority = 3
	})

	MenuCallbackHandler.menu_cursor_reset_callback = function(self, item)
		local colors_to_reset = {red = true, green = true, blue = true}
		local default_color = 0.7

		local scale_to_reset = {scale = true}
		local default_scale = 1

		MenuHelper:ResetItemsToDefaultValue(item, colors_to_reset, default_color)
		MenuHelper:ResetItemsToDefaultValue(item, scale_to_reset, default_scale)
	end

	MenuHelper:AddDivider({
		id = "divider3",
		size = 24,
		menu_id = menu_id_main,
		priority = 2
	})

	MenuHelper:AddButton({
		id = "reset",
		title = "menu_cursor_reset",
		desc = "menu_cursor_reset_desc",
		callback = "menu_cursor_reset_callback",
		menu_id = menu_id_main,
		priority = 1
	})

end)

Hooks:Add("MenuManagerBuildCustomMenus", "LegendaryCursorSkins_MenuManagerBuildCustomMenus", function(menu_manager, nodes)
  nodes[menu_id_main] = MenuHelper:BuildMenu(menu_id_main, { area_bg = "half" })
  MenuHelper:AddMenuItem(nodes["blt_options"], menu_id_main, "menu_legendary_cursor_skins", "menu_legendary_cursor_skins_desc")
end)