MousePointerManager._last_cursor = "default"
MousePointerManager._current_cursor = "default"

MousePointerManager._current_cursor_data = {}
MousePointerManager._current_cursor_defaulted_data = {}

MousePointerManager._user_properties = {
	color = Color(1, 0.7, 0.7, 0.7),
	scale = 1
}
MousePointerManager._user_properties_need_update = false

MousePointerManager._cursor_types = {
	arrow = {0, 0, 19, 23},
	link = {20, 0, 19, 23},
	hand = {40, 0, 19, 23},
	grab = {60, 0, 19, 23}
}

Hooks:PostHook(MousePointerManager, "_setup", "LegendaryCursorSkins_Setup", function(self)
	self:set_cursor(LegendaryCursorSkins.settings.id)
	self:set_user_properties(Color(1, LegendaryCursorSkins.settings.red, LegendaryCursorSkins.settings.green, LegendaryCursorSkins.settings.blue), LegendaryCursorSkins.settings.scale)
end)

function MousePointerManager:set_cursor(cursor_id)
	self._current_cursor = cursor_id or "default"

	self:get_cursor_data(cursor_id)

	self._user_properties_need_update = true
end

function MousePointerManager:set_user_properties(color, scale)
	self._user_properties.color = color or Color(1, 0.7, 0.7, 0.7)
	self._user_properties.scale = scale or 1

	self._user_properties_need_update = true
end

function MousePointerManager:get_cursor_data(cursor_id)
	self._current_cursor_data = tweak_data.cursors[cursor_id] or tweak_data.cursors.default
	self._current_cursor_defaulted_data = clone(self._current_cursor_data)

	self._current_cursor_defaulted_data.texture = self._current_cursor_defaulted_data.texture or "guis/textures/mouse_pointer"

	self._current_cursor_defaulted_data.size = self._current_cursor_defaulted_data.size or {}
	self._current_cursor_defaulted_data.size.width = self._current_cursor_defaulted_data.size.width or 19
	self._current_cursor_defaulted_data.size.height = self._current_cursor_defaulted_data.size.height or 23

	local scale = 23/self._current_cursor_defaulted_data.size.height
	if self._current_cursor_defaulted_data.size.width > self._current_cursor_defaulted_data.size.height then
		scale = 19/self._current_cursor_defaulted_data.size.width
	end

	self._current_cursor_defaulted_data.size.pointer_width = self._current_cursor_defaulted_data.size.width * scale
	self._current_cursor_defaulted_data.size.pointer_height = self._current_cursor_defaulted_data.size.height * scale

	self._current_cursor_defaulted_data.point_offset = self._current_cursor_defaulted_data.point_offset or {}
	self._current_cursor_defaulted_data.point_offset.x = (self._current_cursor_defaulted_data.point_offset.x or 7) * scale
	self._current_cursor_defaulted_data.point_offset.y = (self._current_cursor_defaulted_data.point_offset.y or 2) * scale

	self._current_cursor_defaulted_data.frames = self._current_cursor_defaulted_data.frames or 1
	self._current_cursor_defaulted_data.frame_time = self._current_cursor_defaulted_data.frame_time or 0.1
end

MousePointerManager._last_cursor_frame = 0
function MousePointerManager:update(t, dt)
	if self._current_cursor ~= self._last_cursor then
		if self._current_cursor_data then
			local pointer = self._mouse:child("pointer")

			pointer:set_x(-self._current_cursor_defaulted_data.point_offset.x)
			pointer:set_y(-self._current_cursor_defaulted_data.point_offset.y)

			local width = self._current_cursor_defaulted_data.size.width
			local height = self._current_cursor_defaulted_data.size.height

			MousePointerManager._cursor_types = {
				arrow = {0, 0, 19, 23},
				link = {20, 0, 19, 23},
				hand = {40, 0, 19, 23},
				grab = {60, 0, 19, 23}
			}

			self._cursor_types = {
				arrow =	{0,				0, width, height},
				link =	{width+1,		0, width, height},
				hand =	{(width+1)*2,	0, width, height},
				grab =	{(width+1)*3,	0, width, height}
			}

			pointer:set_width(self._current_cursor_defaulted_data.size.pointer_width)
			pointer:set_height(self._current_cursor_defaulted_data.size.pointer_height)

			pointer:set_image(self._current_cursor_defaulted_data.texture)

			self:set_pointer_frame(0)
		end

		self._last_cursor = self._current_cursor
	end

	if self._user_properties_need_update then
		local pointer = self._mouse:child("pointer")

		pointer:set_width(self._current_cursor_defaulted_data.size.pointer_width * self._user_properties.scale)
		pointer:set_height(self._current_cursor_defaulted_data.size.pointer_height * self._user_properties.scale)

		pointer:set_x(-self._current_cursor_defaulted_data.point_offset.x * self._user_properties.scale)
		pointer:set_y(-self._current_cursor_defaulted_data.point_offset.y * self._user_properties.scale)

		pointer:set_color(self._user_properties.color)

		self._user_properties_need_update = false
	end

	if self._current_cursor_defaulted_data.frames and self._current_cursor_defaulted_data.frames > 1 then
		local frames = self._current_cursor_defaulted_data.frames
		local frame_time = self._current_cursor_defaulted_data.frame_time
		local current_frame = math.floor(t / frame_time) % frames

		if current_frame ~= self._last_cursor_frame then
			self:set_pointer_frame(current_frame)

			self._last_cursor_frame = current_frame
		end
	end
end

function MousePointerManager:set_pointer_frame(frame)
	local height = self._current_cursor_defaulted_data.size.height

	self._cursor_types.arrow[2]	= frame * (height+1)
	self._cursor_types.link[2]	= frame * (height+1)
	self._cursor_types.hand[2]	= frame * (height+1)
	self._cursor_types.grab[2]	= frame * (height+1)

	local rect = self._cursor_types[self._mouse_pointer_image or "arrow"]
	self._mouse:child("pointer"):set_texture_rect(rect[1], rect[2], rect[3], rect[4])
end

function MousePointerManager:set_pointer_image(type)
	local rect = self._cursor_types[type]
	if rect and self._mouse_pointer_image ~= type then
		self._mouse_pointer_image = type

		self._mouse:child("pointer"):set_texture_rect(rect[1], rect[2], rect[3], rect[4])
	end
end